<?php

namespace Ems\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\Services\AvaliacaoService;
use Ems\Util\ExpectedException;

class AvaliacaoController {
    /** @var AvaliacaoService */
    protected $avaliacaoService;

    public function __construct(
        AvaliacaoService $avaliacaoService
    ) {
        $this->avaliacaoService = $avaliacaoService;
    }

    public function getAvaliacoes(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $cdOperador = (string) $row['CDOPERADOR'];

            $avaliacoes = $this->avaliacaoService->getAvaliacoes(
                $cdOperador
            );

            $response->addDataSet(new DataSet('getAvaliacoes', $avaliacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getQuestoesAvaliacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrParticipAval = (int) $row['NRPARTICIPAVAL'];

            $questoes = $this->avaliacaoService->getQuestoesAvaliacao(
                $nrParticipAval
            );

            $response->addDataSet(new DataSet('getQuestoesAvaliacao', $questoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function salvarAvaliacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();

            $this->avaliacaoService->salvarAvaliacao($row);

            $dataSet = [
                'success' => true,
            ];

            $response->addDataSet(new DataSet('salvarAvaliacao', $dataSet));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
