<?php

namespace Ems\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\Services\CertisignService;
use Ems\Util\ExpectedException;

class CertisignController {
    /** @var CertisignService */
    protected $certisignService;

    public function __construct(
        CertisignService $certisignService
    ) {
        $this->certisignService = $certisignService;
    }

    public function certisignUpload(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $base64 = (string) $row['base64'];
            $fileName = (string) $row['fileName'];

            $uploadId = $this->certisignService->upload(
                $fileName, $base64
            );

            $dataset = [
                'uploadId' => $uploadId,
            ];

            $response->addDataSet(new DataSet('certisignUpload', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function certisignCreate(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $response = $this->certisignService->create(
                $row
            );

            $response->addDataSet(new DataSet('certisignCreate', $response));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function certisignDelete(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $id = (int) $row['id'];

            $this->certisignService->delete($id);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
