<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;

class LancamentoLoteRepository {
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function findOneLote(int $nrOrg, int $nrLote): array {
        $params = [
            'P_NRORG'  => $nrOrg,
            'P_NRLOTE' => $nrLote
        ];

        $sql = <<<SQL
            SELECT *
            FROM FPA_LOTELANCEVENTO LOTE
            WHERE LOTE.nrorg = :P_NRORG
              AND LOTE.NRLOTELANCEVENTO = :P_NRLOTE
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $lote = $std->fetchAll()[0];

        return $lote;
    }

    public function getLotesLancamento(array $params) : array {
        $sql = <<<SQL
            SELECT NRLOTELANCEVENTO,
                   DSLOTELANCEVENTO,
                   TO_CHAR(DTLOTELANCEVENTO, 'DD/MM/YYYY') DTLOTELANCEVENTO,
                   TO_CHAR(DTCOMPETLOTEEVE, 'DD/MM/YYYY') DTCOMPETLOTEEVE,
                   TO_CHAR(DTOCORRENCIA, 'DD/MM/YYYY') DTOCORRENCIA,
                   TO_CHAR(DTINCLUSAO, 'DD/MM/YYYY') DTINCLUSAO,
                   IDSTATUSLOTELANC,
                   IDATIVO,
                   NRTIPOMOVIMENT,
                   NRORG
            FROM fpa_lotelancevento LOTE
            WHERE LOTE.nrorg = :P_NRORG
              AND LOTE.dtcompetloteeve BETWEEN :P_INIPERIODO AND :P_FIMPERIODO
              AND LOTE.CDOPERINCLUSAO = :P_CDOPERADOR
            ORDER BY LOTE.DTLOTELANCEVENTO DESC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $lotes = $std->fetchAll();

        return $lotes;
    }

    public function getLancamentosEvento(array $params) : array {
        $sql = <<<SQL
            SELECT
                il.*,
                vm.nmvinculom,
                vm.nrtipovinculom,
                em.nreventom,
                eh.nmeventoh,
                null AS nmestrutgeren,
                vh.nrestrutgeren
            FROM
                fpa_itemlotelanc   il
                JOIN gpe_vinculom       vm ON il.nrvinculom = vm.nrvinculom
                                        AND il.nrorg = vm.nrorg
                JOIN gpe_vinculoh       vh ON vm.nrvinculom = vh.nrvinculom
                                        AND vm.nrorg = vh.nrorg
                                        AND vh.dtmescompetenc = (
                    SELECT
                        MAX(dtmescompetenc)
                    FROM
                        gpe_vinculoh
                    WHERE
                        nrorg = VH.nrorg
                        AND nrvinculom = vh.nrvinculom
                )
                JOIN FPA_EVENTOM        em ON em.nreventom = il.nreventom
                                    AND ( em.nrorg = il.nrorg
                                            OR em.nrorg = :P_NRORGPADRAO )
                JOIN fpa_eventoh        eh ON em.nreventom = eh.nreventom
                                    AND eh.nrorg = em.nrorg
                                    AND eh.dtmescompetenc = (
                    SELECT
                        MAX(dtmescompetenc)
                    FROM
                        fpa_eventoh
                    WHERE
                        nrorg = eh.nrorg
                        AND nreventom = eh.nreventom
                )
            WHERE
                il.nrorg = :P_NRORG
                AND nrlotelancevento = :P_NRLOTE
            ORDER BY VM.NMVINCULOM, EM.NREVENTOM
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $lancamentos = $std->fetchAll();

        return $lancamentos;
    }

    public function getTiposMovimento(array $params) : array {
        $sql = <<<SQL
            SELECT *
            FROM FPA_TIPOMOVIMENT
            WHERE NRORG IN (:P_NRORG, :P_NRORGPADRAO)
            ORDER BY NMTIPOMOVIMENT
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tiposMov = $std->fetchAll();

        return $tiposMov;
    }

    public function insertItemLote(array $params) : bool {
        $sql = <<<SQL
            INSERT INTO FPA_ITEMLOTELANC(
                NRITLOTELANC, NRLOTELANCEVENTO, NRVINCULOM, NREVENTOM,
                VRREFFOLHALOTE, VREVENTOLOTE, IDATIVO, NRORGINCLUSAO,
                DTINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, DTULTATU,
                CDOPERULTATU, NRORG)
            VALUES(
                :P_NRITEM, :P_NRLOTE, :P_NRVINCULO, :P_NREVENTO,
                NULL, NULL, 'S', :P_NRORG,
                SYSDATE, :P_CDOPERADOR, :P_NRORG, SYSDATE,
                :P_CDOPERADOR, :P_NRORG)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return true;
    }

    public function getMissingEventosInVinculoLote(array $params): array {
        $sql = <<<SQL
            SELECT EVM.NREVENTOM
            FROM FPA_EVENTOM EVM
            WHERE EVM.NRORG IN (:P_NRORG, :P_NRORGPADRAO)
              AND EVM.NREVENTOM IN (
                    SELECT NREVENTOM
                    FROM FPA_EVENTOUNIDADE EV_UN
                    WHERE EV_UN.NRORG = :P_NRORG
                      AND EV_UN.NRESTRUTURAM = (
                            SELECT VH.NRESTRUTGEREN
                            FROM GPE_VINCULOM VM
                            JOIN GPE_VINCULOH VH
                                ON VH.NRORG = VM.NRORG
                                   AND VH.NRVINCULOM = VM.NRVINCULOM
                                   AND VH.DTMESCOMPETENC = (
                                        SELECT MAX(VHH.DTMESCOMPETENC)
                                        FROM GPE_VINCULOH VHH
                                        WHERE VHH.NRORG = VH.NRORG
                                              AND VHH.NRVINCULOM = VH.NRVINCULOM
                                   )
                            WHERE VM.NRORG = EV_UN.NRORG
                              AND VM.NRVINCULOM = :P_NRVINCULO
                        )
                )
            AND EVM.NREVENTOM NOT IN (
                    SELECT ITEM.NREVENTOM
                    FROM fpa_itemlotelanc ITEM
                    WHERE ITEM.NRORG = :P_NRORG
                      AND ITEM.NRVINCULOM = :P_NRVINCULO
                      AND ITEM.NRLOTELANCEVENTO = :P_NRLOTE)
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $eventos = $std->fetchAll();

        return $eventos;
    }

    public function insertLoteLancamento(array $params): bool {
        $sql = <<<SQL
            INSERT INTO FPA_LOTELANCEVENTO(
                NRLOTELANCEVENTO, DSLOTELANCEVENTO, DTLOTELANCEVENTO,
                DTCOMPETLOTEEVE, DTOCORRENCIA, IDSTATUSLOTELANC,
                IDATIVO, NRORGINCLUSAO, DTINCLUSAO,
                CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU,
                NRORG, NRTIPOMOVIMENT, IDORIGLOTELANC)
            VALUES(
                :P_NRLOTE, :P_DSLOTE, SYSDATE,
                :P_DTCOMP, :P_DTOCOR, 'DIGITACAO',
                'S', :P_NRORG, SYSDATE,
                :P_CDOPERADOR, :P_NRORG, :P_CDOPERADOR,
                :P_NRORG, :P_NRTIPOMOV, 'TELADELANCAMENTO')
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return true;
    }

    public function getEventosLancamento(array $params) : array {
        $sql = <<<SQL
            SELECT *
            FROM FPA_EVENTOH EVH
            WHERE EVH.NRORG IN (:P_NRORG, :P_NRORGPADRAO)
              AND EVH.NREVENTOH = (
                  SELECT MAX(EVH1.NREVENTOH)
                  FROM FPA_EVENTOH EVH1
                  WHERE EVH1.NRORG IN (:P_NRORG, :P_NRORGPADRAO)
                    AND EVH1.NREVENTOM = EVH.NREVENTOM)
              AND EVH.NREVENTOM IN (
                    SELECT NREVENTOM
                    FROM FPA_EVENTOUNIDADE EV_UN
                    WHERE EV_UN.NRORG = :P_NRORG
                      AND (INSTR(:P_NRESTRUTURAS, ',' || EV_UN.NRESTRUTURAM || ',')) > 0 )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $eventos = $std->fetchAll();

        return $eventos;
    }

    public function editLoteLancamento(array $params): bool {
        $sql = <<<SQL
            UPDATE FPA_LOTELANCEVENTO
            SET DSLOTELANCEVENTO = :P_DSLOTE,
                DTLOTELANCEVENTO = SYSDATE,
                DTOCORRENCIA = :P_DTOCOR,
                DTCOMPETLOTEEVE = :P_DTCOMP,
                NRORGULTATU = :P_NRORG,
                CDOPERULTATU = :P_CDOPERADOR,
                NRTIPOMOVIMENT = :P_NRTIPOMOV
            WHERE NRLOTELANCEVENTO = :P_NRLOTE
              AND NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return true;
    }

    public function confirmLoteLancamento(array $params): bool {
        $sql = <<<SQL
            UPDATE FPA_LOTELANCEVENTO
            SET IDSTATUSLOTELANC = 'PENDENTE'
            WHERE NRLOTELANCEVENTO = :P_NRLOTE
              AND NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return true;
    }

    public function editLancamento(array $params): bool {
        $sql = <<<SQL
            UPDATE FPA_ITEMLOTELANC
            SET VREVENTOLOTE = :P_VALOR,
                VRREFFOLHALOTE = :P_REFERENCIA,
                CDOPERULTATU = :P_CDOPERADOR,
                DTULTATU = SYSDATE
            WHERE NRITLOTELANC = :P_NRLANCAMENTO
              AND NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return true;
    }

    public function deleteLancamentos(array $params): bool {
        $sql = <<<SQL
            DELETE FROM FPA_ITEMLOTELANC
            WHERE NRLOTELANCEVENTO = :P_NRLOTE
              AND NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return true;
    }

    public function deleteLoteLancamento(array $params): bool {
        $sql = <<<SQL
            DELETE FROM FPA_LOTELANCEVENTO
            WHERE NRLOTELANCEVENTO = :P_NRLOTE
              AND NRORG = :P_NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return true;
    }
}
